@@ -1,18 +1,20 @@
 var classtvg_1_1Paint =
 [
-    [ "blend", "classtvg_1_1Paint.html#a4bd3efb580822b2db64034e3f3cd18ad", null ],
-    [ "blend", "classtvg_1_1Paint.html#a7e1214a5dc2f3c19ddeedc5371c4e488", null ],
+    [ "blend", "classtvg_1_1Paint.html#a1878016c09373a24de7b8d618d7be6ee", null ],
     [ "bounds", "classtvg_1_1Paint.html#aed5a6ad8edd827a378111b590ed7755d", null ],
     [ "bounds", "classtvg_1_1Paint.html#a1b51b7344eac48d285c073bc488197d3", null ],
+    [ "clip", "classtvg_1_1Paint.html#a750662e70fca25a8a0e3f6661021c35c", null ],
     [ "composite", "classtvg_1_1Paint.html#aefe4ade5b1b6bc13d518bd5f968af3ee", null ],
     [ "composite", "classtvg_1_1Paint.html#a2e2d077b820d1cfd6502a86e0221638b", null ],
     [ "duplicate", "classtvg_1_1Paint.html#a71afac69fe1e7c352460be5878812635", null ],
-    [ "identifier", "classtvg_1_1Paint.html#a37466e9eff2de62ff15c9df3d72b656d", null ],
+    [ "identifier", "classtvg_1_1Paint.html#aee92a597f61d9bbdbd223b5dfc3877b7", null ],
     [ "opacity", "classtvg_1_1Paint.html#a2ee7517b8bfad47e0f999ba3450e0e3e", null ],
     [ "opacity", "classtvg_1_1Paint.html#a2f25b71fed3ae390eb8051c7ea65aebf", null ],
     [ "rotate", "classtvg_1_1Paint.html#a7841fa0c14643e09735d48042ead68f3", null ],
     [ "scale", "classtvg_1_1Paint.html#a5c057876185525c5723215f95a3984f2", null ],
     [ "transform", "classtvg_1_1Paint.html#ad1e49b6eb81e87bd860f19e1b92c574b", null ],
     [ "transform", "classtvg_1_1Paint.html#ac9f6ba428afe884bcc2861de0a2a96f4", null ],
-    [ "translate", "classtvg_1_1Paint.html#ae896f56bfa51c98a50eee3f2bb62e41e", null ]
+    [ "translate", "classtvg_1_1Paint.html#ae896f56bfa51c98a50eee3f2bb62e41e", null ],
+    [ "type", "classtvg_1_1Paint.html#aec24b20e43f4edf0fc046517db603181", null ],
+    [ "id", "classtvg_1_1Paint.html#abaabdc509cdaba7df9f56c6c76f3ae19", null ]
 ];
\ No newline at end of file
