@@ -3,7 +3,7 @@ var searchData=
   ['b_14',['b',['../structtvg_1_1Fill_1_1ColorStop.html#a4313c9563516f94387762ab05763456b',1,'tvg::Fill::ColorStop']]],
   ['background_15',['background',['../classtvg_1_1Saver.html#a8e3b775984e506f816d6998ca12fd3fb',1,'tvg::Saver']]],
   ['bevel_16',['Bevel',['../group__ThorVG.html#ggaba8b7236c41a171289aef2f3c71eef51a1b9ae4ca6d43fc984af318046e1e7bb5',1,'tvg']]],
-  ['blend_17',['blend',['../classtvg_1_1Paint.html#a7e1214a5dc2f3c19ddeedc5371c4e488',1,'tvg::Paint::blend(BlendMethod method) const noexcept'],['../classtvg_1_1Paint.html#a4bd3efb580822b2db64034e3f3cd18ad',1,'tvg::Paint::blend() const noexcept']]],
+  ['blend_17',['blend',['../classtvg_1_1Paint.html#a1878016c09373a24de7b8d618d7be6ee',1,'tvg::Paint']]],
   ['blendmethod_18',['BlendMethod',['../group__ThorVG.html#ga4dd2f3776540c10b565e0446aa8e9e02',1,'tvg']]],
   ['bounds_19',['bounds',['../classtvg_1_1Paint.html#aed5a6ad8edd827a378111b590ed7755d',1,'tvg::Paint::bounds(float *x, float *y, float *w, float *h) const noexcept'],['../classtvg_1_1Paint.html#a1b51b7344eac48d285c073bc488197d3',1,'tvg::Paint::bounds(float *x, float *y, float *w, float *h, bool transformed) const noexcept']]],
   ['butt_20',['Butt',['../group__ThorVG.html#gga6239974a858100e129f1e0b8ffac4f96ab2635ed1075287dea1eb1598a90df1fe',1,'tvg']]]
