@@ -139,7 +139,7 @@ private:
 
     void buildComp(LottieComposition* comp, float frameNo, LottieExpression* exp);
     void buildComp(jerry_value_t context, float frameNo, LottieLayer* comp, LottieExpression* exp);
-    void buildGlobal(LottieExpression* exp);
+    void buildGlobal(float frameNo, LottieExpression* exp);
 
     //global object, attributes, methods
     jerry_value_t global;
