@@ -39,13 +39,31 @@ void LottieLoader::run(unsigned tid)
     } else {
         LottieParser parser(content, dirName);
         if (!parser.parse()) return;
-        comp = parser.comp;
+        {
+            ScopedLock lock(key);
+            comp = parser.comp;
+        }
+        if (parser.slots) {
+            override(parser.slots, true);
+            parser.slots = nullptr;
+        }
         builder->build(comp);
+
+        release();
     }
     rebuild = false;
 }
 
 
+void LottieLoader::release()
+{
+    if (copy) {
+        free((char*)content);
+        content = nullptr;
+    }
+}
+
+
 /************************************************************************/
 /* External Class Implementation                                        */
 /************************************************************************/
@@ -58,14 +76,15 @@ LottieLoader::LottieLoader() : FrameModule(FileType::Lottie), builder(new Lottie
 
 LottieLoader::~LottieLoader()
 {
-    this->done();
+    done();
 
-    if (copy) free((char*)content);
-    free(dirName);
+    release();
 
     //TODO: correct position?
     delete(comp);
     delete(builder);
+
+    free(dirName);
 }
 
 
@@ -73,12 +92,14 @@ bool LottieLoader::header()
 {
     //A single thread doesn't need to perform intensive tasks.
     if (TaskScheduler::threads() == 0) {
+        LoadModule::read();
         run(0);
         if (comp) {
             w = static_cast<float>(comp->w);
             h = static_cast<float>(comp->h);
             frameDuration = comp->duration();
             frameCnt = comp->frameCnt();
+            frameRate = comp->frameRate;
             return true;
         } else {
             return false;
@@ -88,7 +109,6 @@ bool LottieLoader::header()
     //Quickly validate the given Lottie file without parsing in order to get the animation info.
     auto startFrame = 0.0f;
     auto endFrame = 0.0f;
-    auto frameRate = 0.0f;
     uint32_t depth = 0;
 
     auto p = content;
@@ -182,11 +202,14 @@ bool LottieLoader::header()
 bool LottieLoader::open(const char* data, uint32_t size, bool copy)
 {
     if (copy) {
-        content = (char*)malloc(size);
+        content = (char*)malloc(size + 1);
         if (!content) return false;
         memcpy((char*)content, data, size);
+        const_cast<char*>(content)[size] = '\0';
     } else content = data;
 
+    this->dirName = strdup(".");
+
     this->size = size;
     this->copy = copy;
 
@@ -196,6 +219,7 @@ bool LottieLoader::open(const char* data, uint32_t size, bool copy)
 
 bool LottieLoader::open(const string& path)
 {
+#ifdef THORVG_FILE_IO_SUPPORT
     auto f = fopen(path.c_str(), "r");
     if (!f) return false;
 
@@ -209,11 +233,7 @@ bool LottieLoader::open(const string& path)
 
     auto content = (char*)(malloc(sizeof(char) * size + 1));
     fseek(f, 0, SEEK_SET);
-    auto ret = fread(content, sizeof(char), size, f);
-    if (ret < size) {
-        fclose(f);
-        return false;
-    }
+    size = fread(content, sizeof(char), size, f);
     content[size] = '\0';
 
     fclose(f);
@@ -223,6 +243,9 @@ bool LottieLoader::open(const string& path)
     this->copy = true;
 
     return header();
+#else
+    return false;
+#endif
 }
 
 
@@ -246,10 +269,10 @@ bool LottieLoader::resize(Paint* paint, float w, float h)
 
 bool LottieLoader::read()
 {
-    if (!content || size == 0) return false;
-
     //the loading has been already completed
-    if (comp || !LoadModule::read()) return true;
+    if (!LoadModule::read()) return true;
+
+    if (!content || size == 0) return false;
 
     TaskScheduler::request(this);
 
@@ -267,44 +290,44 @@ Paint* LottieLoader::paint()
 }
 
 
-bool LottieLoader::override(const char* slot)
+bool LottieLoader::override(const char* slots, bool byDefault)
 {
-    if (!comp) done();
-
-    if (!comp || comp->slots.count == 0) return false;
-
-    auto success = true;
+    if (!ready() || comp->slots.count == 0) return false;
 
     //override slots
-    if (slot) {
+    if (slots) {
         //Copy the input data because the JSON parser will encode the data immediately.
-        auto temp = strdup(slot);
+        auto temp = byDefault ? slots : strdup(slots);
 
         //parsing slot json
         LottieParser parser(temp, dirName);
+        parser.comp = comp;
 
         auto idx = 0;
+        auto succeed = false;
         while (auto sid = parser.sid(idx == 0)) {
+            auto applied = false;
             for (auto s = comp->slots.begin(); s < comp->slots.end(); ++s) {
                 if (strcmp((*s)->sid, sid)) continue;
-                if (!parser.apply(*s)) success = false;
+                if (parser.apply(*s, byDefault)) succeed = applied = true;
                 break;
             }
+            if (!applied) parser.skip(sid);
             ++idx;
         }
-
-        if (idx < 1) success = false;
-        free(temp);
-        rebuild = overriden = success;
+        free((char*)temp);
+        rebuild = succeed;
+        overridden |= succeed;
+        return rebuild;
     //reset slots
-    } else if (overriden) {
+    } else if (overridden) {
         for (auto s = comp->slots.begin(); s < comp->slots.end(); ++s) {
             (*s)->reset();
         }
-        overriden = false;
+        overridden = false;
         rebuild = true;
     }
-    return success;
+    return true;
 }
 
 
@@ -314,7 +337,7 @@ bool LottieLoader::frame(float no)
 
     //This ensures that the target frame number is reached.
     frameNo *= 10000.0f;
-    frameNo = roundf(frameNo);
+    frameNo = nearbyintf(frameNo);
     frameNo *= 0.0001f;
 
     //Skip update if frame diff is too small.
@@ -351,17 +374,13 @@ float LottieLoader::curFrame()
 float LottieLoader::duration()
 {
     if (segmentBegin == 0.0f && segmentEnd == 1.0f) return frameDuration;
-
-    if (!comp) done();
-    if (!comp) return 0.0f;
-
-    return frameCnt * (segmentEnd - segmentBegin) / comp->frameRate;
+    return frameCnt * (segmentEnd - segmentBegin) / frameRate;
 }
 
 
 void LottieLoader::sync()
 {
-    this->done();
+    done();
 
     if (rebuild) run(0);
 }
@@ -369,16 +388,13 @@ void LottieLoader::sync()
 
 uint32_t LottieLoader::markersCnt()
 {
-    if (!comp) done();
-    if (!comp) return 0;
-    return comp->markers.count;
+    return ready() ? comp->markers.count : 0;
 }
 
 
 const char* LottieLoader::markers(uint32_t index)
 {
-    if (!comp) done();
-    if (!comp || index >= comp->markers.count) return nullptr;
+    if (!ready() || index >= comp->markers.count) return nullptr;
     auto marker = comp->markers.begin() + index;
     return (*marker)->name;
 }
@@ -386,9 +402,8 @@ const char* LottieLoader::markers(uint32_t index)
 
 bool LottieLoader::segment(const char* marker, float& begin, float& end)
 {
-    if (!comp) done();
-    if (!comp) return false;
-    
+    if (!ready() || comp->markers.count == 0) return false;
+
     for (auto m = comp->markers.begin(); m < comp->markers.end(); ++m) {
         if (!strcmp(marker, (*m)->name)) {
             begin = (*m)->time / frameCnt;
@@ -398,3 +413,15 @@ bool LottieLoader::segment(const char* marker, float& begin, float& end)
     }
     return false;
 }
+
+
+bool LottieLoader::ready()
+{
+    {
+        ScopedLock lock(key);
+        if (comp) return true;
+    }
+    done();
+    if (comp) return true;
+    return false;
+}
\ No newline at end of file
