@@ -63,17 +63,17 @@ void LottieSlot::assign(LottieObject* target, bool byDefault)
         switch (type) {
             case LottieProperty::Type::Position: {
                 if (copy) pair->prop = new LottiePosition(static_cast<LottieTransform*>(pair->obj)->position);
-                pair->obj->override(&static_cast<LottieTransform*>(target)->position, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieTransform*>(target)->position, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::Point: {
                 if (copy) pair->prop = new LottiePoint(static_cast<LottieTransform*>(pair->obj)->scale);
-                pair->obj->override(&static_cast<LottieTransform*>(target)->scale, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieTransform*>(target)->scale, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::Float: {
                 if (copy) pair->prop = new LottieFloat(static_cast<LottieTransform*>(pair->obj)->rotation);
-                pair->obj->override(&static_cast<LottieTransform*>(target)->rotation, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieTransform*>(target)->rotation, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::Opacity: {
@@ -81,27 +81,27 @@ void LottieSlot::assign(LottieObject* target, bool byDefault)
                     if (pair->obj->type == LottieObject::Type::Transform) pair->prop = new LottieOpacity(static_cast<LottieTransform*>(pair->obj)->opacity);
                     else pair->prop = new LottieOpacity(static_cast<LottieSolid*>(pair->obj)->opacity);
                 }
-                pair->obj->override(&static_cast<LottieSolid*>(target)->opacity, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieSolid*>(target)->opacity, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::Color: {
                 if (copy) pair->prop = new LottieColor(static_cast<LottieSolid*>(pair->obj)->color);
-                pair->obj->override(&static_cast<LottieSolid*>(target)->color, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieSolid*>(target)->color, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::ColorStop: {
                 if (copy) pair->prop = new LottieColorStop(static_cast<LottieGradient*>(pair->obj)->colorStops);
-                pair->obj->override(&static_cast<LottieGradient*>(target)->colorStops, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieGradient*>(target)->colorStops, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::TextDoc: {
                 if (copy) pair->prop = new LottieTextDoc(static_cast<LottieText*>(pair->obj)->doc);
-                pair->obj->override(&static_cast<LottieText*>(target)->doc, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieText*>(target)->doc, shallow, !copy);
                 break;
             }
             case LottieProperty::Type::Image: {
                 if (copy) pair->prop = new LottieBitmap(static_cast<LottieImage*>(pair->obj)->data);
-                pair->obj->override(&static_cast<LottieImage*>(target)->data, shallow, byDefault);
+                pair->obj->override(&static_cast<LottieImage*>(target)->data, shallow, !copy);
                 break;
             }
             default: break;
@@ -197,6 +197,14 @@ float LottieTextRange::factor(float frameNo, float totalLen, float idx)
 }
 
 
+void LottieFont::prepare()
+{
+    if (!data.b64src || !name) return;
+
+    Text::load(name, data.b64src, data.size, "ttf", false);
+}
+
+
 void LottieImage::prepare()
 {
     LottieObject::type = LottieObject::Image;
@@ -204,13 +212,9 @@ void LottieImage::prepare()
     auto picture = Picture::gen().release();
 
     //force to load a picture on the same thread
-    TaskScheduler::async(false);
-
     if (data.size > 0) picture->load((const char*)data.b64Data, data.size, data.mimeType, false);
     else picture->load(data.path);
 
-    TaskScheduler::async(true);
-
     picture->size(data.width, data.height);
     PP(picture)->ref();
 
@@ -221,13 +225,11 @@ void LottieImage::prepare()
 void LottieImage::update()
 {
     //Update the picture data
-    TaskScheduler::async(false);
     for (auto p = pooler.begin(); p < pooler.end(); ++p) {
         if (data.size > 0) (*p)->load((const char*)data.b64Data, data.size, data.mimeType, false);
         else (*p)->load(data.path);
         (*p)->size(data.width, data.height);
     }
-    TaskScheduler::async(true);
 }
 
 
@@ -424,7 +426,7 @@ void LottieGroup::prepare(LottieObject::Type type)
 
         /* Figure out if this group is a simple path drawing.
            In that case, the rendering context can be sharable with the parent's. */
-        if (allowMerge && (child->type == LottieObject::Group || !child->mergeable())) allowMerge = false;
+        if (allowMerge && !child->mergeable()) allowMerge = false;
 
         //Figure out this group has visible contents
         switch (child->type) {
@@ -531,7 +533,7 @@ void LottieLayer::prepare(RGB24* color)
 
 float LottieLayer::remap(LottieComposition* comp, float frameNo, LottieExpressions* exp)
 {
-    if (timeRemap.frames || timeRemap.value) {
+    if (timeRemap.frames || timeRemap.value >= 0.0f) {
         frameNo = comp->frameAtTime(timeRemap(frameNo, exp));
     } else {
         frameNo -= startFrame;
@@ -568,7 +570,7 @@ LottieComposition::~LottieComposition()
     for (auto s = slots.begin(); s < slots.end(); ++s) {
         delete(*s);
     }
-    
+
     for (auto m = markers.begin(); m < markers.end(); ++m) {
         delete(*m);
     }
