@@ -37,36 +37,41 @@ struct LottieStroke
 {
     struct DashAttr
     {
-        //0: offset, 1: dash, 2: gap
-        LottieFloat value[3] = {0.0f, 0.0f, 0.0f};
+        LottieFloat offset = 0.0f;
+        LottieFloat* values = nullptr;
+        uint8_t size = 0;
+        uint8_t allocated = 0;
     };
 
     virtual ~LottieStroke()
     {
+        if (dashattr) delete[] dashattr->values;
         delete(dashattr);
     }
 
-    LottieFloat& dash(int no)
+
+    LottieFloat& dashValue()
     {
         if (!dashattr) dashattr = new DashAttr;
-        return dashattr->value[no];
+
+        if (dashattr->size + 1 > dashattr->allocated) {
+            dashattr->allocated = dashattr->size + 2;
+            auto newValues = new LottieFloat[dashattr->allocated];
+            for (uint8_t i = 0; i < dashattr->size; ++i) {
+                newValues[i].copy(dashattr->values[i]);
+            }
+            delete[] dashattr->values;
+            dashattr->values = newValues;
+        }
+
+        return dashattr->values[dashattr->size++];
     }
 
-    float dashOffset(float frameNo, LottieExpressions* exps)
-    {
-        return dash(0)(frameNo, exps);
-    }
 
-    float dashGap(float frameNo, LottieExpressions* exps)
+    LottieFloat& dashOffset()
     {
-        return dash(2)(frameNo, exps);
-    }
-
-    float dashSize(float frameNo, LottieExpressions* exps)
-    {
-        auto d = dash(1)(frameNo, exps);
-        if (d == 0.0f) return 0.1f;
-        else return d;
+        if (!dashattr) dashattr = new DashAttr;
+        return dashattr->offset;
     }
 
     LottieFloat width = 0.0f;
@@ -210,7 +215,7 @@ struct LottieObject
     {
     }
 
-    virtual void override(LottieProperty* prop, bool shallow, bool byDefault)
+    virtual void override(LottieProperty* prop, bool shallow, bool release)
     {
         TVGERR("LOTTIE", "Unsupported slot type");
     }
@@ -253,6 +258,13 @@ struct LottieTextRange
     enum Shape : uint8_t { Square = 1, RampUp, RampDown, Triangle, Round, Smooth };
     enum Unit : uint8_t { Percent = 1, Index };
 
+    LottieTextRange()
+    {
+        style.flags.fillColor = 0;
+        style.flags.strokeColor = 0;
+        style.flags.strokeWidth = 0;
+    }
+
     ~LottieTextRange()
     {
         free(interpolator);
@@ -270,6 +282,11 @@ struct LottieTextRange
         LottieOpacity fillOpacity = 255;
         LottieOpacity strokeOpacity = 255;
         LottieOpacity opacity = 255;
+        struct {
+            bool fillColor : 1;
+            bool strokeColor : 1;
+            bool strokeWidth : 1;
+        } flags;
     } style;
 
     LottieFloat offset = 0.0f;
@@ -287,12 +304,28 @@ struct LottieTextRange
     bool expressible = false;
 
     float factor(float frameNo, float totalLen, float idx);
+
+    void color(float frameNo, RGB24& fillColor, RGB24& strokeColor, float factor, LottieExpressions* exps)
+    {
+        if (style.flags.fillColor) {
+            auto color = style.fillColor(frameNo, exps);
+            fillColor.rgb[0] = tvg::lerp<uint8_t>(fillColor.rgb[0], color.rgb[0], factor);
+            fillColor.rgb[1] = tvg::lerp<uint8_t>(fillColor.rgb[1], color.rgb[1], factor);
+            fillColor.rgb[2] = tvg::lerp<uint8_t>(fillColor.rgb[2], color.rgb[2], factor);
+        }
+        if (style.flags.strokeColor) {
+            auto color = style.strokeColor(frameNo, exps);
+            strokeColor.rgb[0] = tvg::lerp<uint8_t>(strokeColor.rgb[0], color.rgb[0], factor);
+            strokeColor.rgb[1] = tvg::lerp<uint8_t>(strokeColor.rgb[1], color.rgb[1], factor);
+            strokeColor.rgb[2] = tvg::lerp<uint8_t>(strokeColor.rgb[2], color.rgb[2], factor);
+        }
+    }
 };
 
 
 struct LottieFont
 {
-    enum Origin : uint8_t { Local = 0, CssURL, ScriptURL, FontURL, Embedded };
+    enum Origin : uint8_t {Local = 0, CssURL, ScriptURL, FontURL};
 
     ~LottieFont()
     {
@@ -300,14 +333,23 @@ struct LottieFont
         free(style);
         free(family);
         free(name);
+        free(data.b64src);
     }
 
+    struct {
+        char* b64src = nullptr;
+        uint32_t size = 0;
+    } data;
+
     Array<LottieGlyph*> chars;
     char* name = nullptr;
     char* family = nullptr;
     char* style = nullptr;
+    size_t dataSize = 0;
     float ascent = 0.0f;
-    Origin origin = Embedded;
+    Origin origin = Local;
+
+    void prepare();
 };
 
 struct LottieMarker
@@ -349,7 +391,7 @@ struct LottieText : LottieObject, LottieRenderPooler<tvg::Shape>
     }
 
     LottieTextDoc doc;
-    LottieFont* font;
+    LottieFont* font = nullptr;
     Array<LottieTextRange*> ranges;
 
     ~LottieText()
@@ -511,6 +553,12 @@ struct LottieTransform : LottieObject
         LottieFloat y = 0.0f;
     };
 
+    SeparateCoord* separateCoord()
+    {
+        if (!coords) coords = new SeparateCoord;
+        return coords;
+    }
+
     struct RotationEx
     {
         LottieFloat x = 0.0f;
@@ -530,8 +578,7 @@ struct LottieTransform : LottieObject
 
     bool mergeable() override
     {
-        if (!opacity.frames && opacity.value == 255) return true;
-        return false;
+        return true;
     }
 
     LottieProperty* property(uint16_t ix) override
@@ -550,26 +597,26 @@ struct LottieTransform : LottieObject
         return nullptr;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault) override
+    void override(LottieProperty* prop, bool shallow, bool release) override
     {
         switch (prop->type) {
             case LottieProperty::Type::Position: {
-                if (byDefault) position.release();
+                if (release) position.release();
                 position.copy(*static_cast<LottiePosition*>(prop), shallow);
                 break;
             }
             case LottieProperty::Type::Float: {
-                if (byDefault) rotation.release();
+                if (release) rotation.release();
                 rotation.copy(*static_cast<LottieFloat*>(prop), shallow);
                 break;
             }
             case LottieProperty::Type::Point: {
-                if (byDefault) scale.release();
+                if (release) scale.release();
                 scale.copy(*static_cast<LottiePoint*>(prop), shallow);
                 break;
             }
             case LottieProperty::Type::Opacity: {
-                if (byDefault) opacity.release();
+                if (release) opacity.release();
                 opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
                 break;
             }
@@ -615,16 +662,15 @@ struct LottieSolidStroke : LottieSolid, LottieStroke
     {
         if (width.ix == ix) return &width;
         if (dashattr) {
-            if (dashattr->value[0].ix == ix) return &dashattr->value[0];
-            if (dashattr->value[1].ix == ix) return &dashattr->value[1];
-            if (dashattr->value[2].ix == ix) return &dashattr->value[2];
+            for (uint8_t i = 0; i < dashattr->size ; ++i)
+                if (dashattr->values[i].ix == ix) return &dashattr->values[i];
         }
         return LottieSolid::property(ix);
     }
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault) override
+    void override(LottieProperty* prop, bool shallow, bool release) override
     {
-        if (byDefault) color.release();
+        if (release) color.release();
         color.copy(*static_cast<LottieColor*>(prop), shallow);
     }
 };
@@ -637,13 +683,13 @@ struct LottieSolidFill : LottieSolid
         LottieObject::type = LottieObject::SolidFill;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault) override
+    void override(LottieProperty* prop, bool shallow, bool release) override
     {
         if (prop->type == LottieProperty::Type::Opacity) {
-            if (byDefault) opacity.release();
+            if (release) opacity.release();
             opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
         } else if (prop->type == LottieProperty::Type::Color) {
-            if (byDefault) color.release();
+            if (release) color.release();
             color.copy(*static_cast<LottieColor*>(prop), shallow);
         }
     }
@@ -682,9 +728,9 @@ struct LottieGradient : LottieObject
         return nullptr;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
+    void override(LottieProperty* prop, bool shallow, bool release = false) override
     {
-        if (byDefault) colorStops.release();
+        if (release) colorStops.release();
         colorStops.copy(*static_cast<LottieColorStop*>(prop), shallow);
         prepare();
     }
@@ -724,9 +770,8 @@ struct LottieGradientStroke : LottieGradient, LottieStroke
     {
         if (width.ix == ix) return &width;
         if (dashattr) {
-            if (dashattr->value[0].ix == ix) return &dashattr->value[0];
-            if (dashattr->value[1].ix == ix) return &dashattr->value[1];
-            if (dashattr->value[2].ix == ix) return &dashattr->value[2];
+            for (uint8_t i = 0; i < dashattr->size ; ++i)
+                if (dashattr->values[i].ix == ix) return &dashattr->values[i];
         }
         return LottieGradient::property(ix);
     }
@@ -737,9 +782,9 @@ struct LottieImage : LottieObject, LottieRenderPooler<tvg::Picture>
 {
     LottieBitmap data;
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
+    void override(LottieProperty* prop, bool shallow, bool release = false) override
     {
-        if (byDefault) data.release();
+        if (release) data.release();
         data.copy(*static_cast<LottieBitmap*>(prop), shallow);
         update();
     }
@@ -853,7 +898,7 @@ struct LottieLayer : LottieGroup
 
     char* name = nullptr;
     LottieLayer* parent = nullptr;
-    LottieFloat timeRemap = 0.0f;
+    LottieFloat timeRemap = -1.0f;
     LottieLayer* comp = nullptr;  //Precompositor, current layer is belonges.
     LottieTransform* transform = nullptr;
     Array<LottieMask*> masks;
@@ -916,7 +961,7 @@ struct LottieSlot
     void assign(LottieObject* target, bool byDefault);
     void reset();
 
-    LottieSlot(char* sid, LottieObject* obj, LottieProperty::Type type) : sid(sid), type(type)
+    LottieSlot(LottieLayer* layer, LottieObject* parent, char* sid, LottieObject* obj, LottieProperty::Type type) : context{layer, parent}, sid(sid), type(type)
     {
         pairs.push({obj});
     }
@@ -930,9 +975,15 @@ struct LottieSlot
         }
     }
 
+    struct {
+        LottieLayer* layer;
+        LottieObject* parent;
+    } context;
+
     char* sid;
     Array<Pair> pairs;
     LottieProperty::Type type;
+
     bool overridden = false;
 };
 
