@@ -86,8 +86,12 @@ public:
     void addBindResource(const GlBindingResource& binding);
     void setDrawRange(uint32_t offset, uint32_t count);
     void setViewport(const RenderRegion& viewport);
+    void setDrawDepth(int32_t depth) { mDrawDepth = static_cast<float>(depth); }
+    virtual void normalizeDrawDepth(int32_t maxDepth) { mDrawDepth /= static_cast<float>(maxDepth);  }
 
     GlProgram* getProgram() { return mProgram; }
+    const RenderRegion& getViewport() const { return mViewport; }
+    float getDrawDepth() const { return mDrawDepth; }
 private:
     GlProgram* mProgram;
     RenderRegion mViewport = {};
@@ -95,6 +99,7 @@ private:
     uint32_t mIndexCount = {};
     Array<GlVertexLayout> mVertexLayout = {};
     Array<GlBindingResource> mBindingResources = {};
+    float mDrawDepth = 0.f;
 };
 
 class GlStencilCoverTask : public GlRenderTask
@@ -105,6 +110,7 @@ public:
 
     void run() override;
 
+    void normalizeDrawDepth(int32_t maxDepth) override;
 private:
     GlRenderTask* mStencilTask;
     GlRenderTask* mCoverTask;
@@ -121,6 +127,8 @@ public:
 
     void run() override;
 
+    void setRenderSize(uint32_t width, uint32_t height) { mRenderWidth = width; mRenderHeight = height; }
+
 protected:
     GLuint getTargetFbo() { return mTargetFbo; }
 
@@ -128,11 +136,13 @@ protected:
 
     GLuint getResolveFboId();
 
-    virtual void onResolve();
+    void onResolve();
 private:
     GLuint mTargetFbo;
     GlRenderTarget* mFbo;
     Array<GlRenderTask*> mTasks;
+    uint32_t mRenderWidth = 0;
+    uint32_t mRenderHeight = 0;
 };
 
 class GlBlitTask : public GlComposeTask
@@ -141,26 +151,73 @@ public:
     GlBlitTask(GlProgram*, GLuint target, GlRenderTarget* fbo, Array<GlRenderTask*>&& tasks);
     ~GlBlitTask() override = default;
 
-    void setSize(uint32_t width, uint32_t height);
-
     void run() override;
 
-protected:
-    void onResolve() override {}
+    GLuint getColorTexture() const { return mColorTex; }
 
+    void setTargetViewport(const RenderRegion& vp) { mTargetViewport = vp; }
 private:
-    uint32_t mWidth = 0;
-    uint32_t mHeight = 0;
+    GLuint mColorTex = 0;
+    RenderRegion mTargetViewport = {};
 };
 
 class GlDrawBlitTask : public GlComposeTask
 {
 public:
     GlDrawBlitTask(GlProgram*, GLuint target, GlRenderTarget* fbo, Array<GlRenderTask*>&& tasks);
-    ~GlDrawBlitTask() override = default;
+    ~GlDrawBlitTask() override;
+
+    void setPrevTask(GlRenderTask* task) { mPrevTask = task; }
+
+    void setParentSize(uint32_t width, uint32_t height) { mParentWidth = width; mParentHeight = height; }
 
     void run() override;
+
+private:
+    GlRenderTask* mPrevTask = nullptr;
+    uint32_t mParentWidth = 0;
+    uint32_t mParentHeight = 0;
 };
 
+class GlClipTask : public GlRenderTask
+{
+public:
+    GlClipTask(GlRenderTask* clip, GlRenderTask* mask);
+    ~GlClipTask() override;
+
+    void run() override;
+
+    void normalizeDrawDepth(int32_t maxDepth) override;
+private:
+    GlRenderTask* mClipTask;
+    GlRenderTask* mMaskTask;
+};
+
+class GlSimpleBlendTask : public GlRenderTask
+{
+public:
+    GlSimpleBlendTask(BlendMethod method, GlProgram* program);
+    ~GlSimpleBlendTask() override = default;
+
+    void run() override;
+private:
+    BlendMethod mBlendMethod;
+};
+
+class GlComplexBlendTask: public GlRenderTask
+{
+public:
+    GlComplexBlendTask(GlProgram* program, GlRenderTarget* dstFbo, GlRenderTarget* dstCopyFbo, GlRenderTask* stencilTask, GlComposeTask* composeTask);
+    ~GlComplexBlendTask() override;
+
+    void run() override;
+
+    void normalizeDrawDepth(int32_t maxDepth) override;
+private:
+    GlRenderTarget* mDstFbo;
+    GlRenderTarget* mDstCopyFbo;
+    GlRenderTask* mStencilTask;
+    GlComposeTask* mComposeTask;
+};
 
 #endif /* _TVG_GL_RENDER_TASK_H_ */
